//***********************************************************************
//
//                              JHFPParametersFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

//------------------------------------------------------------------------------
public class JHFPParametersFrame extends JFISDialog
{
  private Component Parent;
  private JLabel jLabelDistance = new JLabel();
  private JComboBox jCBDistance = new JComboBox();
  private DefaultComboBoxModel jCBDistanceModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.DistanceType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JCheckBox jCBSimplifiedDistance = new JCheckBox();
  private JLabel jLabelNmfDist = new JLabel();
  private IntegerField jIFNmfDist = new IntegerField();
  private JLabel jLabelNmfVar = new JLabel();
  private IntegerField jIFNmfVar = new IntegerField();
  private JLabel jLabelMerging = new JLabel();
  private JComboBox jCBMerging = new JComboBox();
  private DefaultComboBoxModel jCBMergingModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.MergingType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JLabel jLabelHeteroPenalty = new JLabel();
  private DoubleField jDFHeteroPenalty = new DoubleField();
  private JLabel jLabelHeteroProportion = new JLabel();
  private DoubleField jDFHeteroProportion = new DoubleField();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonCancel = new JButton();
  private JButton jButtonApply = new JButton();
  private JFISHFP hfp;
//------------------------------------------------------------------------------
  public JHFPParametersFrame( Component parent, JFISHFP hfp )
  {
    this.hfp = hfp;
    this.Parent = parent;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.getContentPane().setLayout(new GridBagLayout());
    this.getContentPane().add(jLabelDistance,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    for( int i=0 ; i<JFISHFP.DistanceType().length ; i++ )
      this.jCBDistanceModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jCBDistance.setModel(this.jCBDistanceModel);
    this.getContentPane().add(jCBDistance,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    this.getContentPane().add(jCBSimplifiedDistance,   new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 0), 0, 0));
    this.getContentPane().add(jIFNmfDist,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jLabelNmfDist,   new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(jLabelNmfVar,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(jIFNmfVar,  new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jLabelMerging,  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    for( int i=0 ; i<JFISHFP.MergingType().length ; i++ )
      this.jCBMergingModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jCBMerging.setModel(this.jCBMergingModel);
    this.getContentPane().add(jCBMerging,   new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    this.getContentPane().add(jLabelHeteroPenalty,  new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(jDFHeteroPenalty,  new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jLabelHeteroProportion,  new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(jDFHeteroProportion,   new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jPanelValidation,   new GridBagConstraints(0, 7, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(10, 5, 5, 5), 0, 0));
    jPanelValidation.add(jButtonCancel,    new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonApply,   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // init des objets
    Object [] params = this.hfp.GetParameters();
    String [] types = JFISHFP.DistanceType();
    for( int i=0 ; i<types.length ; i++ )
      if( params[0].equals(types[i]) )
        this.jCBDistance.setSelectedIndex(i);
    this.jCBSimplifiedDistance.setSelected(((Boolean)params[1]).booleanValue());
    this.jIFNmfDist.setValue(((Integer)params[2]).intValue());
    this.jIFNmfVar.setValue(((Integer)params[3]).intValue());
    types = JFISHFP.MergingType();
    for( int i=0 ; i<types.length ; i++ )
      if( params[4].equals(types[i]) )
        this.jCBMerging.setSelectedIndex(i);
    this.jDFHeteroPenalty.setValue(((Double)params[5]).doubleValue());
    this.jDFHeteroProportion.setValue(((Double)params[6]).doubleValue());

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setResizable(false);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("HFPParameters"));
    jLabelDistance.setText(LocaleFIS.GetString("Distance") + ":");
    jCBSimplifiedDistance.setText(LocaleFIS.GetString("SimplifiedDistance"));
    jLabelNmfDist.setText(LocaleFIS.GetString("DistanceMFNb") + ":");
    jLabelNmfVar.setText(LocaleFIS.GetString("VarianceMFNb") + ":");
    jLabelMerging.setText(LocaleFIS.GetString("MergingCriterion") + ":");
    jLabelHeteroPenalty.setText(LocaleFIS.GetString("HeteroPenalty") + ":");
    jLabelHeteroProportion.setText(LocaleFIS.GetString("HeteroProportion") + ":");
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
  }

//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try { this.hfp.SetParameters(this.jCBDistance.getSelectedIndex(), this.jCBSimplifiedDistance.isSelected(), this.jIFNmfDist.getValue(), this.jIFNmfVar.getValue(), this.jCBMerging.getSelectedIndex(), this.jDFHeteroPenalty.getValue(), this.jDFHeteroProportion.getValue()); }
  catch(Throwable t) { MessageFIS.Error(null, t); }
  this.dispose();
  }
}
