##########################################################################
##            MAKEFILE FOR FISPRO                                       ##
##                                                                      ##
##     Usage :                                                          ##
##                                                                      ##
##          Compilation : make all                                      ##
##                                                                      ##
##          Nettoyage : make clean                                      ##
##                                                                      ##
##          Execution : make run                                        ##
##                                                                      ##
##########################################################################

#################################
# Configuration du makefile     #
# Configuration of the Makefile #
#################################
# the following lines are machine dependent
# java path - edit to point to the jdk root directory
JAVA_PATH       := /usr/local/jdk1.6.0_14
#
# path separator: / on linux/unix, \\ on Windows
#SEP		:= \\
SEP		:= /
# classpath separator: : on linux/unix, ; on Windows
#CLASSPATH_SEP   := ;
CLASSPATH_SEP   := :
# rm on linux/unix, del on Windows
#CMD_RM          := del
CMD_RM          := rm

J3D_PATH        :=$(JAVA_PATH)
# end machine dependent lines

PROJECT_PATH       := ./
MAIN_FILE          := fis/MainFIS
LIBRARY_PATH       := ..$(SEP)bin
BIN_CLASS_PATH     := ..$(SEP)bin$(SEP)class
BIN_JAR_PATH       := ..$(SEP)bin$(SEP)jar
FREEHEP_CLASS_PATH := $(BIN_JAR_PATH)$(SEP)freehep-base.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphics2d.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphicsio.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphicsio-emf.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphicsio-gif.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphicsio-pdf.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-graphicsio-ps.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)$(SEP)freehep-j3d.jar
UTIL_CLASS_PATH    := $(BIN_JAR_PATH)$(SEP)util.jar
PLOT_CLASS_PATH    := $(BIN_JAR_PATH)$(SEP)plot.jar
J3D_CLASS_PATH     := $(J3D_PATH)$(SEP)lib$(SEP)ext$(SEP)j3dcore.jar$(CLASSPATH_SEP)$(J3D_PATH)$(SEP)lib$(SEP)ext$(SEP)j3dutils.jar$(CLASSPATH_SEP)$(J3D_PATH)$(SEP)lib$(SEP)ext$(SEP)vecmath.jar
JGRAPH_CLASS_PATH  := $(BIN_JAR_PATH)$(SEP)jgraph.jar
CLASS_PATH         := $(BIN_CLASS_PATH)$(CLASSPATH_SEP)$(FREEHEP_CLASS_PATH)$(CLASSPATH_SEP)$(UTIL_CLASS_PATH)$(CLASSPATH_SEP)$(PLOT_CLASS_PATH)$(CLASSPATH_SEP)$(J3D_CLASS_PATH)$(CLASSPATH_SEP)$(JGRAPH_CLASS_PATH)
PACKAGES           := fis org/freehep/j3d/plot
EMPTY              :=
CMD_JAVA           := $(JAVA_PATH)$(SEP)bin$(SEP)java
CMD_JAVAC          := $(JAVA_PATH)$(SEP)bin$(SEP)javac -nowarn
CMD_JAR            := $(JAVA_PATH)$(SEP)bin$(SEP)jar

########
#############################################################
# Ceci est un truc                                          #
# This is a trick				            #
#############################################################
# those words must be unused words such as kgfh.
CLEAN_CODE := clean_
CMP_CODE   := cmp_

HPACKAGES := $(patsubst %,$(CMP_CODE)%,$(PACKAGES))
CPACKAGES := $(patsubst %,$(CLEAN_CODE)%,$(PACKAGES))

##############
# Les rgles #
# The rules  #
##############
all: build jar
run: build jar execute

#############################
# Compile tous les fichiers #
# Compile all the files     #
#############################
build : $(HPACKAGES)
	@echo $(PACKAGES)
	@echo '===' Compilation Completed

######################################
# Compile les fichiers d'un package  #
# Compile all files in a package     #
# warning: fis and fis.print must be compiled together in one pass
######################################
$(HPACKAGES) :
	@echo Compilation de $(subst $(CMP_CODE),$(EMPTY), $@)
	$(CMD_JAVAC) -d $(BIN_CLASS_PATH) -classpath $(CLASS_PATH) $(PROJECT_PATH)fis/*.java $(PROJECT_PATH)fis/print/*.java $(PROJECT_PATH)org/freehep/j3d/plot/*.java

################################################################
# Cree le fichier jar contenant les classes java	       #
# Create the jar file holding the java classes                 #
################################################################
jar:
	@echo creating fispro.jar
	$(CMD_JAR) cmf MANIFEST.MF $(BIN_JAR_PATH)/fispro.jar
	$(CMD_JAR) uf $(BIN_JAR_PATH)/fispro.jar -C $(BIN_CLASS_PATH) fis -C $(BIN_CLASS_PATH) org -C $(BIN_CLASS_PATH) resources
	@echo fispro.jar completed

################################################################
# Supprime les fichiers non necessaires d'un package  	       #
# Delete the file not needed from a package                    #
################################################################
clean : $(CPACKAGES)
	@echo $(PACKAGES)
	@echo deletion '==='  Completed

$(CPACKAGES) :
	$(CMD_RM) $(BIN_CLASS_PATH)$(SEP)$(subst $(CLEAN_CODE),$(EMPTY),$@)$(SEP)*.class

############################################################
# Lance le fichier dont le nom est donne dans MAIN_FILE    #
# Execute the file the name of which is given in MAIN_FILE #
############################################################
execute:
	@$(CMD_JAVA) -Dfispropath=..$(SEP) -Djava.library.path=$(LIBRARY_PATH) -classpath $(FREEHEP_CLASS_PATH)$(CLASSPATH_SEP)$(UTIL_CLASS_PATH)$(CLASSPATH_SEP)$(PLOT_CLASS_PATH)$(CLASSPATH_SEP)$(J3D_CLASS_PATH)$(CLASSPATH_SEP)$(BIN_JAR_PATH)/fispro.jar$(CLASSPATH_SEP)$(BIN_JAR_PATH)/jgraph.jar $(MAIN_FILE)


